//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	rmd160_sum_base32
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 32 5-bit character encoded values.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(char *) pointer to 16 base32 characters where
//				the checksum result is to be stored.
//
// returns	(char *) pointer to returned result
//		(char *) NULL if error
//-----------------------------------------------------------------------------

#define FUNCTION_NAME "rmd160_sum_base32"

char *
rmd160_sum_base32 (
	RMD160		source_p
,
	char *		result_p
)
{
    static char		encode	[32]	= "0123456789abcdefghijklmnopqrstuv";

    rmd160uint32	temp	[5]	;


    //-- Make sure we have a result buffer.
    if ( ! result_p && ! ( result_p = malloc( 33 ) ) ) return NULL;

    //-- Get the results as words.
    if ( ! rmd160_sum_words( source_p , temp ) ) return NULL;


    //----------------------------------------------------------------------
    // Convert words to characters in the result buffer.  Since this is my
    // own extension to RIPEMD-160, I have chosen to use little-endian order
    // consistently for the entire result.
    //----------------------------------------------------------------------
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]    )) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]>> 5)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]>>10)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]>>15)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]>>20)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[0]>>25)) ];
    * ( result_p ++ ) = encode[ (0x03&(temp[0]>>30)) | (0x1c&(temp[1]<<2)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[1]>> 3)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[1]>> 8)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[1]>>13)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[1]>>18)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[1]>>23)) ];
    * ( result_p ++ ) = encode[ (0x0f&(temp[1]>>28)) | (0x10&(temp[2]<<4)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>> 1)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>> 6)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>>11)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>>16)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>>21)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[2]>>26)) ];
    * ( result_p ++ ) = encode[ (0x01&(temp[2]>>31)) | (0x1e&(temp[3]<<1)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[3]>> 4)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[3]>> 9)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[3]>>14)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[3]>>19)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[3]>>24)) ];
    * ( result_p ++ ) = encode[ (0x07&(temp[3]>>29)) | (0x18&(temp[4]<<3)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>> 2)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>> 7)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>>12)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>>17)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>>22)) ];
    * ( result_p ++ ) = encode[ (0x1f&(temp[4]>>27)) ];
    * ( result_p ++ ) = '\0';

    //-- All done, return.
    return result_p;

}

#undef FUNCTION_NAME
