//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	rmd160_sum_words
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 5 32-bit words.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(rmd160uint32 *) pointer to 5 words of 32 bits where
//				the checksum result is to be stored.
//
// returns	(rmd160uint32 *) pointer to returned result
//		(rmd160uint32 *) NULL if error
//-----------------------------------------------------------------------------

#define FUNCTION_NAME "rmd160_sum_words"

rmd160uint32 *
rmd160_sum_words (
	RMD160		arg_handle
,
	rmd160uint32 *	arg_result_p
)
{
    struct rmd160_object	work	;


    //-- If no source RMD160 object then return an error.
    if ( ! arg_handle ) return NULL;

    //-- Make sure we have a result buffer.
    if ( ! arg_result_p && ! ( arg_result_p = malloc( 5 * sizeof (rmd160uint32) ) ) ) {
	return NULL;
    }

    //-- Make a copy of the calculated state and work from there.
    rmd160_copy( & work , arg_handle );

    //-- Zero out the unused words.
    {
	rmd160uint32 * p;
	p = work.wptr;
	if ( work.bpos ) ++ p;
	while ( p < work.wend ) * ( p ++ ) = 0U;
    }

    //-----------------------------------------------
    // Insert a trailing 1-bit.  This makes data that
    // differs only by 0 padding have different sums.
    //-----------------------------------------------
    * ( work.wptr ) |= ( (rmd160uint32) 0x80 ) << ( work.bpos << 3 );

    //-- Calculate this block now if no space for length.
    if ( ( work.wend - work.wptr ) <= 2 ) {
	_rmd160_calc( work.state , work.data );

	//-- Clear the block for length insert.
	memset( work.data , 0U , 14 * sizeof (rmd160uint32) );
    }

    //-- Insert the length.
    work.data[14] = work.len[0] << 3;
    work.data[15] = ( work.len[1] << 3 ) | ( work.len[0] >> 29 );

    //-- Finally, calculate block with the length.
    _rmd160_calc( work.state , work.data );

    //-- Return state to caller buffer.
    memcpy( arg_result_p , work.state , 5 * sizeof (rmd160uint32) );

    //-- All done, return.
    return arg_result_p;

}
#pragma section end CODE

#undef FUNCTION_NAME
